/*  Copyright (C) 2010-2024 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tandemCalculatePolar.h"
#include "tandemDefaultStrategy.h"
#include "tandemCalculatePolarConfig.h"

#include <moduleBasics/strategySelector.h>
#include <moduleBasics/report.h>
#include <moduleBasics/plot.h>
#include <aixml/node.h>

#include <algorithm>
#include <memory>
#include <exception>
#include <iomanip>
#include <string>
#include <vector>
#include <utility>  // make_pair

#ifdef _WIN32
    #include <fileapi.h>
    #include <windows.h>
#endif  // _WIN32

using std::endl;
using std::exception;
using std::fixed;
using std::ofstream;
using std::setprecision;
using std::string;
using std::stringstream;
using std::vector;

void tandemCalculatePolar::initialize() {
    myRuntimeInfo->out << "Calculate Polar for tandem: Initialize..." << endl;
    config_ = std::make_shared<tandemCalculatePolarConfig>(rtIO_->moduleConfig);
    myDefaultStrategy = std::make_shared<tandemDefaultStrategy>(rtIO_, config_);
    this->selectAeroStrategy(); //TODO kayra: suan herhangi bir strategy user ile belirlenmiyor
}

void tandemCalculatePolar::run() {
    myRuntimeInfo->out << "Calculate Polar for tandem: Run..." << endl;
    std::string strategy = "defaultStrategy";
    if (strategy == "defaultStrategy") {
        myDefaultStrategy->run();
    }
}

void tandemCalculatePolar::update() {
    myRuntimeInfo->out << "Calculate Polar for tandem: Update..." << endl;
    if (config_->execution_mode.value() == "mode_0") {
        myDefaultStrategy->writePolarXML();
        myDefaultStrategy->setOutputValues();
        myDefaultStrategy->setReferenceValues();
    } else if (config_->execution_mode.value() == "mode_1") {
        myDefaultStrategy->setReferenceValues();
    } else {
        myRuntimeInfo->err << "execution_mode: " << config_->execution_mode.value() << " in config file program_settings/execution_mode is not supported! " << endl;
    }
    myDefaultStrategy->update_xml_entries(config_->execution_mode.value());
}

void tandemCalculatePolar::report() {
    myRuntimeInfo->out << "Calculate Polar for tandem: Report..." << endl;
    myDefaultStrategy->writePolarData();
    myRuntimeInfo->out << "Calculate Polar for tandem: Report..." << endl;
    myDefaultStrategy->writeViscDragData();
}

void tandemCalculatePolar::save() {
    myRuntimeInfo->out << "Calculate Polar for tandem: Save..." << endl;
}

void tandemCalculatePolar::selectAeroStrategy() {
}
