/*  Copyright (C) 2010-2024 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CALCULATEPOLAR_SRC_TANDEM_TANDEMCALCULATEPOLAR_H_
#define CALCULATEPOLAR_SRC_TANDEM_TANDEMCALCULATEPOLAR_H_

#include "tandemCalculatePolarConfig.h"
#include "tandemDefaultStrategy.h"

#include <moduleBasics/module.h>
#include <moduleBasics/strategySelector.h>
#include <moduleBasics/report.h>
#include <moduleBasics/plot.h>

#include <memory>
#include <vector>
#include <string>

class tandemCalculatePolar : public Strategy {
 public:
    std::string polarXMLFilePath;          /**< Path to the polar XML output file, derived from the I/O file path */

    explicit tandemCalculatePolar(std::shared_ptr<RuntimeIO> rtIO): rtIO_(rtIO), report_(rtIO) {}
    ~tandemCalculatePolar() = default;
    void initialize();
    void run();
    void update();
    void report();
    void save();
    void selectAeroStrategy();

 private:
    std::shared_ptr<RuntimeIO> rtIO_; /*<-- RuntimeIO object*/
    std::shared_ptr<tandemCalculatePolarConfig> config_;
    std::shared_ptr<tandemDefaultStrategy> myDefaultStrategy;
    Report report_; /*Report object*/
};

#endif  // CALCULATEPOLAR_SRC_TANDEM_TANDEMCALCULATEPOLAR_H_
